#!/usr/bin/env python
# Example Device Under Test

import sys
import re
import os
import subprocess
import ConfigParser

regexRowCount = re.compile(r'.*COUNT.*\n([0-9]+)', re.MULTILINE)

class Config:
    """ Runtime configuration for this DUT, used to remember
        the path of the MySQL client """

    def __init__(self):
        # Test whether we can find the portable XAMPP installation on Windows
        if self.is_exe("FH-Datenbanken_xampp-portable/mysql/bin/mysql.exe"):
            self.mysql = "FH-Datenbanken_xampp-portable/mysql/bin/mysql.exe"
        else:
            self.mysql = self.findInPath("mysql");
            if not self.mysql:
                config = ConfigParser.ConfigParser()
                config.read("./db.cfg")
                self.mysql = config.get("paths", "mysql");


    def is_exe(self, fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
    
    def findInPath(self, program):
        """ Attempts to find the given program in path.
            Returns the absolute path if found. """
        fpath, fname = os.path.split(program)
        if fpath:
            if self.is_exe(program):
                return program
        else:
            for path in os.environ["PATH"].split(os.pathsep):
                path = path.strip('"')
                exe_file = os.path.join(path, program)
                if self.is_exe(exe_file):
                    return exe_file

        return None

class SqlEntry:
    """ A single executable SQL statement """

    def __init__(self, config, section, task, expectedRows):
        self.config  = config
        self.section = int(section)
        self.task    = int(task)
        self.expectedRows = int(expectedRows)
        self.sqlStatement = ""

    def execute(self, sqlStatement):
        """ Executes the given statement """

        # Start the MySQL client using the appropriate
        # database and user. Verbosity is set to "-v -v"
        # because this seems to be the only mode that
        # prints the number of rows in the result set
        PIPE = subprocess.PIPE
        proc = subprocess.Popen([self.config.mysql, '-u', 'fh_abbild', 'fh_abbild'], stdout=PIPE, stdin=PIPE)
            
        return(proc.communicate(input = sqlStatement)[0])

    def getResultSet(self):
        """ Executes the statement and prints the result to stderr """  

        procRes = self.execute(self.sqlStatement);
        return (procRes)

    def getLineCount(self):
        
        # Prepare SQL statement and send it to the MySQL
        # process via its STDIN
        sqlStatement = "SELECT COUNT(*) FROM \n(\n%s) temp; " % self.sqlStatement.replace(";", "")      

        procRes = self.execute(sqlStatement);
        
        # Search for the line containing the result set
        count = 0;
        lineRes = regexRowCount.search(procRes)

        if lineRes:
            count = int(lineRes.group(1))

        return (count);


    def addStatementLine(self, line):
        """ Adds the given line to this SQL statement if its 
            not a comment and not empty """
        strippedLine = line.strip()
        
        # Make sure the stripped line isn't empty and
        # isn't a comment
        if strippedLine \
            and not strippedLine.startswith('--') \
            and not strippedLine == '\n':
            
            self.sqlStatement += line;

    def isValidStatement(self):
        """ Very basic validation of the SQL string """
        return (bool(self.sqlStatement.strip()))

    def __str__(self):
        return "Aufgabe %s.%s - Erwarte %s Zeilen\n%s" \
            % (self.section, self.task, self.expectedRows, self.sqlStatement)


if __name__ == "__main__":
    config = Config()

    # First argument is name of executable
    # and is not interesting
    argv = sys.argv
    argv.pop(0)



    relevantSection = int(argv.pop(0));
    relevantTask = int(argv.pop(0));

    regex = re.compile(r'^--.*Aufgabe.*([0-9]+)\.([0-9]+).*\[([0-9]+).*\].*$')
    
    # Holds all entries that were parsed in the SQL file
    entries = []

    with open('db-ws13.sql', 'r') as sqlFile:
        # Parse file line by line and search for navigatable 
        # comments
        for line in sqlFile:
            match = regex.search(line)

            if (match):
                entries.append(
                    SqlEntry(config, match.group(1), match.group(2), match.group(3))
                )
            elif entries:
                entries[-1].addStatementLine(line)

    # Finding the relevant entry to execute it
    for entry in entries:
        if (entry.section == relevantSection and entry.task == relevantTask):
            foundEntry = True

            # Ensuring the statement is valid
            if not entry.isValidStatement():
                print >> sys.stderr, ("Fehler: SQL Statement ist fehlerhaft")
            else:
                # Run statement and make sure the result set
                # contains the correct number of rows
                count = entry.getLineCount()
                difference = count - entry.expectedRows 

                print entry.getResultSet()

                if difference != 0:
                    print >> sys.stderr, ("Fehler: %s Zeilen, Abweichung um %s Zeilen" % (count, difference))
                else:
                    print ("%s Zeilen, keine Abweichung" % count)

    if not foundEntry:
         print >> sys.stderr, ("Fehler: Entsprechende Query nicht gefunden")